#!/usr/bin/env python

import glob
import json
import os
import uuid
import boto3
import datetime
import random
from urlparse import urlparse
import logging
import mimetypes
import magic

from botocore.client import ClientError

logger = logging.getLogger()
logger.setLevel(logging.INFO)

S3 = boto3.resource('s3')

def handler(event, context):
    mime = magic.Magic(mime=True)

    assetID = str(uuid.uuid4())
    queue_success= os.environ['sqs']
    queue_fail= os.environ['sqs_fail']
    sourceS3Bucket = os.environ['SourceBucket']
    sourceS3Key = event['Records'][0]['s3']['object']['key']
    sourceS3 = 's3://'+ sourceS3Bucket + '/' + sourceS3Key
    desintationBucket = os.environ['DestinationBucket']
    destinationS3 = 's3://' + desintationBucket
    mediaConvertRole = os.environ['MediaConvertRole']
    application = os.environ['Application']
    region = os.environ['AWS_DEFAULT_REGION']
    statusCode = 200
    jobs = []
    job = {}
   
    # Use MediaConvert SDK UserMetadata to tag jobs with the assetID 
    # Events from MediaConvert will have the assetID in UserMedata
    jobMetadata = {}
    jobMetadata['assetID'] = assetID
    jobMetadata['application'] = application
    jobMetadata['input'] = sourceS3
    jobMetadata['key'] = sourceS3Key
    
    
    try:    
        # gete apppathid from source 
        if sourceS3Key != None:
            apppathid = sourceS3Key.split('_')[0]
        
        if not apppathid:
            apppathid = 1
        contentType = mimetypes.guess_type(sourceS3Key)[0]
        logger.info("Content Type %s", contentType)
        
        if contentType != None:
            contentType = contentType.split('/')[0]
        # check file extension
        key_extension  =  os.path.splitext(sourceS3Key)[1]
        
        if contentType != 'video' and key_extension != '.mts' and key_extension != '.wmv':
            logger.info("not VIDEO")
            # copy the content to another bucket
            destkey  = apppathid+"/"+sourceS3Key
            copy_source = {
                            'Bucket': sourceS3Bucket,
                            'Key': sourceS3Key
                        }
            S3.meta.client.copy(copy_source, desintationBucket,destkey)
            logger.info("copied")
            # call to sqs
            sqs = boto3.resource('sqs')

            queue = sqs.get_queue_by_name(QueueName=queue_success)
            outputfilepath = destkey
            messagebody =  {
                        'key':sourceS3Key,
                        "outputfile":outputfilepath
                    }
            # Send message to SQS queue
            response = queue.send_message(
                MessageAttributes={
                    'filename': {
                        'DataType': 'String',
                        'StringValue': sourceS3Key
                    },
                   
                },
                MessageBody=(
                   json.dumps(messagebody)
                )
            )
            logger.info("added in queue")
            return 1
            
        ## in casse of video submit for trancoding    
     
        jobInput = {}
       
        bucket = S3.Bucket(sourceS3Bucket)
        
            
        with open('job.json') as json_data:
            jobInput['filename'] = 'Default'
            logger.info('jobInput: %s', jobInput['filename'])

            jobInput['settings'] = json.load(json_data)
            logger.info(json.dumps(jobInput['settings']))

            jobs.append(jobInput)
             
        # get the account-specific mediaconvert endpoint for this region
        mediaconvert_client = boto3.client('mediaconvert', region_name=region)
        endpoints = mediaconvert_client.describe_endpoints()
        
        # add the account-specific endpoint to the client session 
        client = boto3.client('mediaconvert', region_name=region, endpoint_url=endpoints['Endpoints'][0]['Url'], verify=False)
        
        for j in jobs:
            jobSettings = j['settings']
            jobFilename = j['filename']

            # Save the name of the settings file in the job userMetadata
            jobMetadata['settings'] = jobFilename

            # Update the job settings with the source video from the S3 event 
            jobSettings['Inputs'][0]['FileInput'] = sourceS3

            # Update the job settings with the destination paths for converted videos.  We want to replace the
            # destination bucket of the output paths in the job settings, but keep the rest of the
            # path
            destinationS3 = 's3://' + os.environ['DestinationBucket'] + '/'  + apppathid
                
            mp4file = os.path.splitext(sourceS3Key)[0]
            jobMetadata['outputfile'] = apppathid+"/"+mp4file +'.mp4'   

            for outputGroup in jobSettings['OutputGroups']:
                
                logger.info("outputGroup['OutputGroupSettings']['Type'] == %s", outputGroup['OutputGroupSettings']['Type']) 
                
                if outputGroup['OutputGroupSettings']['Type'] == 'FILE_GROUP_SETTINGS':
                    templateDestination = outputGroup['OutputGroupSettings']['FileGroupSettings']['Destination']
                    templateDestinationKey = urlparse(templateDestination).path
                    logger.info("templateDestinationKey == %s", templateDestinationKey)
                    outputGroup['OutputGroupSettings']['FileGroupSettings']['Destination'] = destinationS3+templateDestinationKey

                elif outputGroup['OutputGroupSettings']['Type'] == 'HLS_GROUP_SETTINGS':
                    templateDestination = outputGroup['OutputGroupSettings']['HlsGroupSettings']['Destination']
                    templateDestinationKey = urlparse(templateDestination).path
                    logger.info("templateDestinationKey == %s", templateDestinationKey)
                    outputGroup['OutputGroupSettings']['HlsGroupSettings']['Destination'] = destinationS3+templateDestinationKey
                
                elif outputGroup['OutputGroupSettings']['Type'] == 'DASH_ISO_GROUP_SETTINGS':
                    templateDestination = outputGroup['OutputGroupSettings']['DashIsoGroupSettings']['Destination']
                    templateDestinationKey = urlparse(templateDestination).path
                    logger.info("templateDestinationKey == %s", templateDestinationKey)
                    outputGroup['OutputGroupSettings']['DashIsoGroupSettings']['Destination'] = destinationS3+templateDestinationKey

                elif outputGroup['OutputGroupSettings']['Type'] == 'DASH_ISO_GROUP_SETTINGS':
                    templateDestination = outputGroup['OutputGroupSettings']['DashIsoGroupSettings']['Destination']
                    templateDestinationKey = urlparse(templateDestination).path
                    logger.info("templateDestinationKey == %s", templateDestinationKey)
                    outputGroup['OutputGroupSettings']['DashIsoGroupSettings']['Destination'] = destinationS3+templateDestinationKey
                
                elif outputGroup['OutputGroupSettings']['Type'] == 'MS_SMOOTH_GROUP_SETTINGS':
                    templateDestination = outputGroup['OutputGroupSettings']['MsSmoothGroupSettings']['Destination']
                    templateDestinationKey = urlparse(templateDestination).path
                    logger.info("templateDestinationKey == %s", templateDestinationKey)
                    outputGroup['OutputGroupSettings']['MsSmoothGroupSettings']['Destination'] = destinationS3+templateDestinationKey
                else:
                    logger.error("Exception: Unknown Output Group Type %s", outputGroup['OutputGroupSettings']['Type'])
                    statusCode = 500
            
            logger.info(json.dumps(jobSettings))

            # Convert the video using AWS Elemental MediaConvert
            job = client.create_job(Role=mediaConvertRole, UserMetadata=jobMetadata, Settings=jobSettings)

    except Exception as e:
        logger.error('Exception: %s', e)
        statusCode = 500
        
        ## udpate in failed queu
        sqs = boto3.resource('sqs')

        queue = sqs.get_queue_by_name(QueueName=queue_fail)
        messagebody =  {
                        'key':sourceS3Key
                    }
            # Send message to SQS queue
        response = queue.send_message(
                MessageAttributes={
                    'filename': {
                        'DataType': 'String',
                        'StringValue': sourceS3Key
                    },
                   
                },
                MessageBody=(
                   json.dumps(messagebody)
                )
            )
        logger.info("added in queue")
        
        raise

    finally:
        return {
            'statusCode': statusCode,
            'body': json.dumps(job, indent=4, sort_keys=True, default=str),
            'headers': {'Content-Type': 'application/json', 'Access-Control-Allow-Origin': '*'}
        }
